const hljs = require("highlight.js");
hljs.registerLanguage('terraform', hljsDefineTerraform);

const MAP = {
    py: "python",
    js: "javascript",
    json: "javascript",
    rb: "ruby",
    csharp: "cs",
};

function normalize(lang) {
    if (!lang) {
        return null;
    }

    const lower = lang.toLowerCase();
    return MAP[lower] || lower;
}

// from: https://github.com/highlightjs/highlightjs-terraform
function hljsDefineTerraform(hljs) {
    const NUMBERS = {
        className: 'number',
        begin: '\\b\\d+(\\.\\d+)?',
        relevance: 0
    };
    const STRINGS = {
        className: 'string',
        begin: '"',
        end: '"',
        contains: [{
            className: 'variable',
            begin: '\\${',
            end: '\\}',
            relevance: 9,
            contains: [{
                className: 'string',
                begin: '"',
                end: '"'
            }, {
                className: 'meta',
                begin: '[A-Za-z_0-9]*' + '\\(',
                end: '\\)',
                contains: [
                    NUMBERS, {
                        className: 'string',
                        begin: '"',
                        end: '"',
                        contains: [{
                            className: 'variable',
                            begin: '\\${',
                            end: '\\}',
                            contains: [{
                                className: 'string',
                                begin: '"',
                                end: '"',
                                contains: [{
                                    className: 'variable',
                                    begin: '\\${',
                                    end: '\\}'
                                }]
                            }, {
                                className: 'meta',
                                begin: '[A-Za-z_0-9]*' + '\\(',
                                end: '\\)'
                            }]
                        }]
                    },
                    'self']
            }]
        }]
    };

    return {
        aliases: ['tf', 'hcl'],
        keywords: 'resource variable provider output locals module data terraform|10',
        literal: 'false true null',
        contains: [
            hljs.COMMENT('\\#', '$'),
            NUMBERS,
            STRINGS
        ]
    };
}

/**
 * @param {string} lang
 * @param {string} code
 * @returns {string|{html: boolean, body}}
 */
function highlight(lang, code) {
    if (!lang)
        return {
            body: code,
            html: false,
        };

    // Normalize lang
    lang = normalize(lang);

    try {
        return hljs.highlight(code, {
            language: lang,
        }).value;
    } catch (e) {
        console.error(e);
    }

    return {
        body: code,
        html: false,
    };
}

module.exports = {
    book: {
        assets: "./css",
        css: ["website.css"],
    },
    ebook: {
        assets: "./css",
        css: ["ebook.css"],
    },
    blocks: {
        code: function (block) {
            return highlight(block.kwargs.language, block.body);
        },
    },
};
